@@ -2,7 +2,7 @@
 # Configuration file for using the XML library in GNOME applications
 #
 XML2_LIBDIR="-L${libdir}"
-XML2_LIBS="-lxml2 -lz -lpthread -L/usr/local/lib -liconv -lm "
+XML2_LIBS="-lxml2 -lz  -L/usr/local/lib -liconv -lm "
 XML2_INCLUDEDIR="-I${includedir}/libxml2 -I/usr/local/include"
-MODULE_VERSION="xml2-2.5.7"
+MODULE_VERSION="xml2-2.5.10"
 
